/*
 * frontier client log message handling
 * 
 * Author: Sergey Kosyakov
 *
 * $Id: frontier_log.c,v 1.5 2007/12/11 15:54:32 dwd Exp $
 *
 *  Copyright (C) 2007  Fermilab
 *
 *  This program is free software: you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public License
 *  as published by the Free Software Foundation, either version 3 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 */
#include "frontier_client/frontier_log.h"
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define LOG_BUF_SIZE	1024*2

static
#ifdef _REENTRANT
__thread
#endif /*_REENTRANT*/
char _frontier_log_msg[LOG_BUF_SIZE];


extern int frontier_log_level;
extern char *frontier_log_file;
extern int frontier_log_dup;

static const char *log_desc[]=
 {
  "debug",
  "warn ",
  "error"
 };


void frontier_log(int level,const char *file,int line,const char *fmt,...)
 {
  int ret;
  int fd;
  va_list ap;
  
  if(level<frontier_log_level) return;
  // To make sure that log_desc is not out of boundaries
  if(level<0) level=0;
  if(level>2) level=2;
  
  ret=snprintf(_frontier_log_msg,LOG_BUF_SIZE-1,"%s [%s:%d]: ",log_desc[level],file,line);
  
  va_start(ap,fmt);
  ret+=vsnprintf(_frontier_log_msg+ret,LOG_BUF_SIZE-ret-1,fmt,ap);
  va_end(ap);

  if(ret>LOG_BUF_SIZE-2)
    ret=LOG_BUF_SIZE-2;
  
  _frontier_log_msg[ret]='\n';
  _frontier_log_msg[ret+1]=0;
  
  if(!frontier_log_file||(frontier_log_dup&&(level>=FRONTIER_LOGLEVEL_WARNING)))
   {
    write(1,_frontier_log_msg,ret+1);
    fsync(1);
    if(!frontier_log_file) return;
   }
  fd=open(frontier_log_file,O_CREAT|O_APPEND|O_WRONLY,0644);
  if(fd<0) return;
  write(fd,_frontier_log_msg,ret+1);
  close(fd);
 }

